@@ -87,7 +87,7 @@ import java.util.StringTokenizer;
  * @author Hein Meling
  * @author <a href="mailto:russel.winder@concertant.com">Russel Winder</a>
  * @author Danno Ferrin
- * @version $Revision: 21452 $
+ * @version $Revision$
  */
 public class Groovyc extends MatchingTask {
     private final LoggingHelper log = new LoggingHelper(this);
@@ -121,12 +121,11 @@ public class Groovyc extends MatchingTask {
     private Javac javac;
     private boolean jointCompilation;
 
-    private List<File> temporaryFiles = new ArrayList(2);
+    private List<File> temporaryFiles = new ArrayList<File>(2);
     private File stubDir;
     private boolean keepStubs;
-    
-    private Set<String> scriptExtensions = new LinkedHashSet<String>();
 
+    private Set<String> scriptExtensions = new LinkedHashSet<String>();
 
     /**
      * Adds a path for source compilation.
@@ -173,50 +172,50 @@ public class Groovyc extends MatchingTask {
     }
 
     /**
-     * Set the extension to use when searching for Groovy source files. 
+     * Set the extension to use when searching for Groovy source files.
      * Accepts extensions in the form *.groovy, .groovy or groovy
      *
      * @param scriptExtension the extension of Groovy source files
      */
     public void setScriptExtension(String scriptExtension) {
-    	if(scriptExtension.startsWith("*.")) {
-    		this.scriptExtension = scriptExtension;
-    	} else if(scriptExtension.startsWith(".")) {
-    		this.scriptExtension = "*" + scriptExtension;
-    	} else {
-    		this.scriptExtension = "*." + scriptExtension;
-    	}
+        if (scriptExtension.startsWith("*.")) {
+            this.scriptExtension = scriptExtension;
+        } else if (scriptExtension.startsWith(".")) {
+            this.scriptExtension = "*" + scriptExtension;
+        } else {
+            this.scriptExtension = "*." + scriptExtension;
+        }
     }
-    
+
     /**
      * Get the extension to use when searching for Groovy source files.
      *
      * @return the extension of Groovy source files
      */
     public String getScriptExtension() {
-    	return scriptExtension;
+        return scriptExtension;
     }
-    
+
     /**
      * Sets the bytecode compatibility mode
-     * 
+     *
      * @param version the bytecode compatibility mode
      */
     public void setTargetBytecode(String version) {
-        if(CompilerConfiguration.PRE_JDK5.equals(version) || CompilerConfiguration.POST_JDK5.equals(version)) {
+        if (CompilerConfiguration.PRE_JDK5.equals(version) || CompilerConfiguration.POST_JDK5.equals(version)) {
             this.targetBytecode = version;
         }
     }
 
     /**
      * Retrieves the compiler bytecode compatibility mode.
-     * 
+     *
      * @return bytecode compatibility mode. Can be either <tt>1.5</tt> or <tt>1.4</tt>.
      */
     public String getTargetBytecode() {
         return this.targetBytecode;
     }
-    
+
     /**
      * Set the destination directory into which the Java source
      * files should be compiled.
@@ -588,7 +587,7 @@ public class Groovyc extends MatchingTask {
         checkParameters();
         resetFileLists();
         loadRegisteredScriptExtensions();
-        
+
         if (javac != null) jointCompilation = true;
 
         // scan source directories and dest directory to build up
@@ -633,10 +632,10 @@ public class Groovyc extends MatchingTask {
         SourceFileScanner sfs = new SourceFileScanner(this);
         File[] newFiles;
         for (String extension : getScriptExtensions()) {
-        	m.setFrom("*." + extension);
-        	m.setTo("*.class");
-        	newFiles = sfs.restrictAsFiles(files, srcDir, destDir, m);
-        	addToCompileList(newFiles);
+            m.setFrom("*." + extension);
+            m.setTo("*.class");
+            newFiles = sfs.restrictAsFiles(files, srcDir, destDir, m);
+            addToCompileList(newFiles);
         }
 
         if (jointCompilation) {
@@ -740,16 +739,20 @@ public class Groovyc extends MatchingTask {
                     Enumeration children = rc.getChildren();
                     while (children.hasMoreElements()) {
                         RuntimeConfigurable childrc = (RuntimeConfigurable) children.nextElement();
-                        if(childrc.getElementTag().equals("compilerarg")) {
+                        if (childrc.getElementTag().equals("compilerarg")) {
                             for (Iterator i = childrc.getAttributeMap().entrySet().iterator(); i.hasNext();) {
                                 final Map.Entry e = (Map.Entry) i.next();
                                 final String key = e.getKey().toString();
-                                if(key.equals("value")) {
+                                if (key.equals("value")) {
                                     final String value = getProject().replaceProperties(e.getValue().toString());
                                     StringTokenizer st = new StringTokenizer(value, " ");
-                                    while(st.hasMoreTokens()) {
+                                    while (st.hasMoreTokens()) {
                                         String optionStr = st.nextToken();
-                                        jointOptions.add(optionStr.replace("-X", "-FX"));
+                                        String replaced = optionStr.replace("-X", "-FX");
+                                        if(optionStr == replaced) {
+                                        	replaced = optionStr.replace("-W", "-FW"); // GROOVY-5063
+                                        }
+                                        jointOptions.add(replaced);
                                     }
                                 }
                             }
@@ -792,12 +795,12 @@ public class Groovyc extends MatchingTask {
                     if ((memoryMaximumSize != null) && !memoryMaximumSize.equals("")) {
                         commandLineList.add("-Xmx" + memoryMaximumSize);
                     }
-                    if(!"*.groovy".equals(getScriptExtension())) {
-                    	String tmpExtension = getScriptExtension();
-                    	if(tmpExtension.startsWith("*.")) tmpExtension = tmpExtension.substring(1);
-                    	commandLineList.add("-Dgroovy.default.scriptExtension=" + tmpExtension);
+                    if (!"*.groovy".equals(getScriptExtension())) {
+                        String tmpExtension = getScriptExtension();
+                        if (tmpExtension.startsWith("*.")) tmpExtension = tmpExtension.substring(1);
+                        commandLineList.add("-Dgroovy.default.scriptExtension=" + tmpExtension);
                     }
-                    commandLineList.add("org.codehaus.groovy.tools.FileSystemCompiler");
+                    commandLineList.add(FileSystemCompilerFacade.class.getName());
                 }
                 commandLineList.add("--classpath");
                 commandLineList.add(classpath.toString());
@@ -862,8 +865,7 @@ public class Groovyc extends MatchingTask {
                     executor.setCommandline(commandLine);
                     try {
                         executor.execute();
-                    }
-                    catch (final IOException ioe) {
+                    } catch (final IOException ioe) {
                         throw new BuildException("Error running forked groovyc.", ioe);
                     }
                     final int returnCode = executor.getExitValue();
@@ -887,8 +889,8 @@ public class Groovyc extends MatchingTask {
 
                         configuration = FileSystemCompiler.generateCompilerConfigurationFromOptions(cli);
                         configuration.setScriptExtensions(getScriptExtensions());
-                    	String tmpExtension = getScriptExtension();
-                    	if(tmpExtension.startsWith("*.")) tmpExtension = tmpExtension.substring(1);
+                        String tmpExtension = getScriptExtension();
+                        if (tmpExtension.startsWith("*.")) tmpExtension = tmpExtension.substring(1);
                         configuration.setDefaultScriptExtension(tmpExtension);
 
                         // Load the file name list
@@ -897,12 +899,12 @@ public class Groovyc extends MatchingTask {
 
                         fileNameErrors = fileNameErrors && !FileSystemCompiler.validateFiles(filenames);
 
-                        if(targetBytecode != null) {
+                        if (targetBytecode != null) {
                             configuration.setTargetBytecode(targetBytecode);
                         }
-                        
+
                         if (!fileNameErrors) {
-                            FileSystemCompiler.doCompilation(configuration, makeCompileUnit(), filenames);
+                            FileSystemCompiler.doCompilation(configuration, makeCompileUnit(), filenames, false);
                         }
 
                     } catch (Exception re) {
@@ -1042,23 +1044,23 @@ public class Groovyc extends MatchingTask {
     public boolean getKeepStubs() {
         return keepStubs;
     }
-    
+
     private Set<String> getScriptExtensions() {
-    	return scriptExtensions;
+        return scriptExtensions;
     }
 
     private void loadRegisteredScriptExtensions() {
-    	if (scriptExtensions.isEmpty()) {
-    		
-    		scriptExtensions.add(getScriptExtension().substring(2)); // first extension will be the one set explicitly on <groovyc>
-    		
-    		Path classpath = getClasspath() != null ? getClasspath() : new Path(getProject());
-    		final String[] pe = classpath.list();
-    		final GroovyClassLoader loader = new GroovyClassLoader(getClass().getClassLoader());
-    		for (String file : pe) {
-    			loader.addClasspath(file);
-    		}
-    		scriptExtensions.addAll(SourceExtensionHandler.getRegisteredExtensions(loader));
-    	}
+        if (scriptExtensions.isEmpty()) {
+
+            scriptExtensions.add(getScriptExtension().substring(2)); // first extension will be the one set explicitly on <groovyc>
+
+            Path classpath = getClasspath() != null ? getClasspath() : new Path(getProject());
+            final String[] pe = classpath.list();
+            final GroovyClassLoader loader = new GroovyClassLoader(getClass().getClassLoader());
+            for (String file : pe) {
+                loader.addClasspath(file);
+            }
+            scriptExtensions.addAll(SourceExtensionHandler.getRegisteredExtensions(loader));
+        }
     }
 }
